# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

class MikeCloudException(Exception):
    """Exception raised by MIKE Cloud Python SDK"""
    def __init__(self, *args: object) -> None:
        super().__init__(*args)

class MikeCloudRestApiException(MikeCloudException):
    """Exception raised when MIKE Cloud REST API call fails"""
    def __init__(self, message, detail, status_code, *args: object) -> None:
        super().__init__(message, *args)
        self._message = message
        self._status_code = status_code
        self._detail = detail
    
    @property
    def message(self):
        return self._message

    @property
    def status_code(self):
        return self._status_code
    
    @property
    def detail(self):
        return self._detail
    
    def __repr__(self) -> str:
        return f"{type(self).__name__}, HTTP ERROR {self.status_code}, {self.message}, {self.detail}"
    
    def __str__(self) -> str:
        return self.__repr__() + " " + super().__str__()